/*------------------------------------------------------------------------/
/  Bitbanging MMCv3/SDv1/SDv2 (in SPI mode) control module
/-------------------------------------------------------------------------/
/
/  Copyright (C) 2012, ChaN, all right reserved.
/
/ * This software is a free software and there is NO WARRANTY.
/ * No restriction on use. You can use, modify and redistribute it for
/   personal, non-profit or commercial products UNDER YOUR RESPONSIBILITY.
/ * Redistributions of source code must retain the above copyright notice.
/
/--------------------------------------------------------------------------/
 Features and Limitations:

 * Very Easy to Port
   It uses only 4 bit of GPIO port. No interrupt, no SPI port is used.

 * Platform Independent
   You need to modify only a few macros to control GPIO ports.

 * Low Speed
   The data transfer rate will be several times slower than hardware SPI.

/-------------------------------------------------------------------------*/
// MicroBlaze porting K.I

#include "diskio.h"             /* Common include file for FatFs and disk I/O layer */

static
DSTATUS Stat = STA_NOINIT;      /* Disk status */

static
BYTE CardType;                  /* b0:MMC, b1:SDv1, b2:SDv2, b3:Block addressing */



/*--------------------------------------------------------------------------

   Public Functions

---------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------*/
/* Get Disk Status                                                       */
/*-----------------------------------------------------------------------*/
DSTATUS disk_status (
    BYTE pdrv                       /* Drive number (always 0) */
)
{
    return (pdrv) ? STA_NODISK : Stat;
}

/*-----------------------------------------------------------------------*/
/* Initialize Disk Drive                                                 */
/*-----------------------------------------------------------------------*/

DSTATUS disk_initialize (
    BYTE pdrv               /* Physical drive nmuber (0) */
)
{
    DSTATUS s;
 
    if (pdrv) return RES_NOTRDY;

    // OpenCores SPI Initialization
    SPI_TRANS_TYPE  = 0x1; //INIT_SD
    SPI_TRANS_CTRL  = 0x1; //Trans Start
    // wait for trans done
    while(SPI_TRANS_STS==1);
    s=SPI_TRANS_ERROR;
    if ((s & 0x03)==0) s = 0;
    else s = STA_NOINIT;

    Stat = s;

    return s;
}

/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */
/*-----------------------------------------------------------------------*/

DRESULT disk_read (
    BYTE pdrv,              /* Physical drive nmuber (0) */
    BYTE *buff,             /* Pointer to the data buffer to store read data */
    DWORD sector,           /* Start sector number (LBA) */
    BYTE count              /* Sector count (1..128) */
)
{
    BYTE s;
    DWORD i,j;
    DWORD sd_adr;

    if (disk_status(pdrv) & STA_NOINIT) return RES_NOTRDY;
    if (!count) return RES_PARERR;
    for (j=0;j<count;j++) {
        SPI_RX_FIFO_CONTROL =0x1;
        // address set
        sd_adr = (sector+j) * 512;
        SPI_SD_ADDR_7_0 = 0xff & sd_adr;
        SPI_SD_ADDR_15_8 = (0xff00 & sd_adr)>>8;
        SPI_SD_ADDR_23_16 = (0xff0000 & sd_adr)>>16;
        SPI_SD_ADDR_31_24 = (0xff000000 & sd_adr)>>24;
        SPI_TRANS_TYPE  = 0x2; //Block Read
        SPI_TRANS_CTRL  = 0x1; //Trans Start
        // wait for trans done
        while(SPI_TRANS_STS==1);
        for(i=0;i<512;i++){
	  // byte read from the FIFO
            *(buff+j*512+i) = SPI_RX_FIFO_DATA;
        }
        s=SPI_TRANS_ERROR;
        if ((s & 0xc)==0) s =RES_OK;
        else {
            s = RES_ERROR;
            break;
        }
    }
    return s;
}

/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */
/*-----------------------------------------------------------------------*/

#if _USE_WRITE
DRESULT disk_write (
    BYTE pdrv,              /* Physical drive nmuber (0) */
    const BYTE *buff,       /* Pointer to the data to be written */
    DWORD sector,           /* Start sector number (LBA) */
    BYTE count              /* Sector count (1..128) */
)
{
    BYTE s;
    DWORD i,j;
    DWORD sd_adr;

    if (disk_status(pdrv) & STA_NOINIT) return RES_NOTRDY;
    if (!count) return RES_PARERR;
    for (j=0;j<count;j++) {
        SPI_TX_FIFO_CONTROL =0x1;
        // byte write to the FIFO
        for(i=0;i<512;i++){
            SPI_TX_FIFO_DATA = *(buff+j*512+i);
        }
        // address set
        sd_adr = (sector+j) * 512;
        SPI_SD_ADDR_7_0 = 0xff & sd_adr;
        SPI_SD_ADDR_15_8 = (0xff00 & sd_adr)>>8;
        SPI_SD_ADDR_23_16 = (0xff0000 & sd_adr)>>16;
        SPI_SD_ADDR_31_24 = (0xff000000 & sd_adr)>>24;
        SPI_TRANS_TYPE  = 0x3; //Block write
        SPI_TRANS_CTRL  = 0x1; //Trans Start
        // wait for trans done
        while(SPI_TRANS_STS==1);
        s=SPI_TRANS_ERROR;
        if ((s & 0x30)==0) s =RES_OK;
        else {
            s = RES_ERROR;
            break;
        }
    }
    return s;
}
#endif
/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */
/*-----------------------------------------------------------------------*/

#if _USE_IOCTL
DRESULT disk_ioctl (
    BYTE drv,               /* Physical drive nmuber (0) */
    BYTE ctrl,              /* Control code */
    void *buff              /* Buffer to send/receive control data */
)
{
    DRESULT res;
    BYTE n, csd[16], *ptr = buff;
    DWORD cs;


    if (drv) return RES_PARERR;
    if (Stat & STA_NOINIT) return RES_NOTRDY;

    return RES_OK;
}
#endif

/*-----------------------------------------------------------------------*/
/* This function is defined for only project compatibility               */

void disk_timerproc (void)
{
        /* Nothing to do */
}

